#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

#define TL_VER_STR_LEN          (8)     // "<0-255>.<0-255>\0"
#define TL_DATE_STR_LEN         (11)    // "<1-12>/<1-31>/<1900-20XX>\0"
#define TL_SERNO_STR_LEN        (17)    // "<0-32767> <0-4294967296>\0"

typedef struct
{
    char serNo       [TL_SERNO_STR_LEN];    // Serial Number
    char serNoDate   [TL_DATE_STR_LEN];     // Serial Number Issue Date
    char appVer      [TL_VER_STR_LEN];      // Appl / Nav Proc SW Version
    char appVerDate  [TL_DATE_STR_LEN];     // Appl / Nav Proc SW Version Date
    char coreVer     [TL_VER_STR_LEN];      // Core / Signal Proc Version
    char coreVerDate [TL_DATE_STR_LEN];     // Core / Signal Version Date

} TL_RCV_INFO;


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle      hnd;
    int                    rc;
    int                    ret     = 0;
    int                    devIdx;
    char                  *devName = DEFAULT_DEV;
    char                   fullDevName[32];
    char                  *pgmname = argv[0];
    TSYNC_ERROR            err     = TSYNC_SUCCESS;
    unsigned int           index;
    TSYNC_ReceiverInfoObj  ri;


    /* If invalid number of arguments... */
    if ((argc != 3) && (argc != 4))
    {
        printf(" Usage: GR_GetRcvInfo <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Rcvr Info message */
    index = atoi(argv[2]);

    // Send Get Rcvr Info transaction
    err = TSYNC_GR_getRcvInfo(hnd, index, &ri);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }
    
         /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) && 
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%s %s %s %s %s %s", 
                ((TL_RCV_INFO*)&(ri.info))->serNo,
                ((TL_RCV_INFO*)&(ri.info))->serNoDate, 
                ((TL_RCV_INFO*)&(ri.info))->appVer,
                ((TL_RCV_INFO*)&(ri.info))->appVerDate,
                ((TL_RCV_INFO*)&(ri.info))->coreVer,
                ((TL_RCV_INFO*)&(ri.info))->coreVerDate);
     }

    printf("\n");
    printf(" GR (%d) Rcv Info (%u bytes):\n", (index), ri.len);
    printf("  -------------------------------\n");
    printf("  Serial #: %s\n", ((TL_RCV_INFO*)&(ri.info))->serNo);
    printf("  Date:     %s\n", ((TL_RCV_INFO*)&(ri.info))->serNoDate);
    printf("  -------------------------------\n");
    printf("  Appl Ver: %s\n", ((TL_RCV_INFO*)&(ri.info))->appVer);
    printf("  Date:     %s\n", ((TL_RCV_INFO*)&(ri.info))->appVerDate);
    printf("  -------------------------------\n");
    printf("  Core Ver: %s\n", ((TL_RCV_INFO*)&(ri.info))->coreVer);
    printf("  Date:     %s\n", ((TL_RCV_INFO*)&(ri.info))->coreVerDate);
    printf("  -------------------------------\n");

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
